const searchData = {"envversion": {"sphinx.domains.c": 2, "sphinx.domains.changeset": 1, "sphinx.domains.citation": 1, "sphinx.domains.cpp": 5, "sphinx.domains.index": 1, "sphinx.domains.javascript": 2, "sphinx.domains.math": 2, "sphinx.domains.python": 3, "sphinx.domains.rst": 2, "sphinx.domains.std": 2, "sphinx.ext.intersphinx": 1, "repo_docs.ext.enhanced_search": 1, "sphinx": 56}, "data": [{"id": 0, "doc_id": 0, "filename": "contents.html", "domain_name": "std", "name": "contents", "display_name": "Contents", "type": "doc", "display_type": "Page", "docname": "contents", "anchor": "", "priority": -1, "content": "Hopper Architecture Compatibility About this Document Application Compatibility on Hopper Architecture Verifying Hopper Compatibility for Existing Applications Applications Built Using CUDA Toolkit 11.7 or Earlier Applications Built Using CUDA Toolkit 11.8 Building Applications with Hopper Architecture Support Building Applications Using CUDA Toolkit 11.7 or Earlier Building Applications Using CUDA Toolkit 11.8 Independent Thread Scheduling Compatibility Revision History Notices Notice OpenCL Trademarks", "keywords": []}, {"id": 1, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#about-this-document", "display_name": "About this Document", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "about-this-document", "priority": -1, "content": "This application note, Hopper Architecture Compatibility Guide for CUDA Applications, is intended to help developers ensure that their NVIDIA \u00ae CUDA \u00ae applications will run on the NVIDIA \u00ae Hopper architecture based GPUs. This document provides guidance to developers who are familiar with programming in CUDA C++ and want to make sure that their software applications are compatible with Hopper architecture.", "keywords": []}, {"id": 2, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#application-compatibility-on-hopper-architecture", "display_name": "Application Compatibility on Hopper Architecture", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "application-compatibility-on-hopper-architecture", "priority": -1, "content": "A CUDA application binary (with one or more GPU kernels) can contain the compiled GPU code in two forms, binary cubin objects and forward-compatible PTX assembly for each kernel. Both cubin and PTX are generated for a certain target compute capability. A cubin generated for a certain compute capability is supported to run on any GPU with the same major revision and same or higher minor revision of compute capability. For example, a cubin generated for compute capability 8.0 is supported to run on a GPU with compute capability 8.6, however a cubin generated for compute capability 8.6 is not supported to run on a GPU with compute capability 8.0, and a cubin generated with compute capability 8.x is not supported to run on a GPU with compute capability 9.0. Kernel can also be compiled to a PTX form. At the application load time, PTX is compiled to cubin and the cubin is used for kernel execution. Unlike cubin, PTX is forward-compatible. Meaning PTX is supported to run on any GPU with compute capability higher than the compute capability assumed for generation of that PTX. For example, PTX code generated for compute capability 8.x is supported to run on compute capability 8.x or any higher revision (major or minor), including compute capability 9.0. Therefore although it is optional, it is recommended that all applications should include PTX of the kernels to ensure forward-compatibility. To read more about cubin and PTX compatibilities see Compilation with NVCC from the CUDA C++ Programming Guide . When a CUDA application launches a kernel on a GPU, the CUDA Runtime determines the compute capability of the GPU in the system and uses this information to find the best matching cubin or PTX version of the kernel. If a cubin compatible with that GPU is present in the binary, the cubin is used as-is for execution. Otherwise, the CUDA Runtime first generates compatible cubin by JIT-compiling 1 the PTX and then the cubin is used for the execution. If neither compatible cubin nor PTX is available, kernel launch results in a failure. Application binaries that include PTX version of kernels, should work as-is on the Hopper GPUs. In such cases, rebuilding the application is not required. However application binaries which do not include PTX (only include cubins), need to be rebuilt to run on the Hopper GPUs. To know more about building compatible applications read Building Applications with Hopper Architecture Support", "keywords": []}, {"id": 3, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#applications-built-using-cuda-toolkit-11-7-or-earlier", "display_name": "Applications Built Using CUDA Toolkit 11.7 or Earlier", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "applications-built-using-cuda-toolkit-11-7-or-earlier", "priority": -1, "content": "CUDA applications built using CUDA Toolkit versions 2.1 through 11.7 are compatible with Hopper GPUs as long as they are built to include PTX versions of their kernels. This can be tested by forcing the PTX to JIT-compile at application load time with following the steps: Download and install the latest driver from https://www.nvidia.com/drivers . Set the environment variable CUDA_FORCE_PTX_JIT=1 . Launch the application. With CUDA_FORCE_PTX_JIT=1 , GPU binary code embedded in an application binary is ignored. Instead PTX code for each kernel is JIT-compiled to produce GPU binary code. An application fails to execute if it does not include PTX. This means the application is not Hopper architecture compatible and needs to be rebuilt for compatibility. On the other hand, if the application works properly with this environment variable set, then the application is Hopper compatible. Be sure to unset the CUDA_FORCE_PTX_JIT environment variable after testing is done.", "keywords": []}, {"id": 4, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#applications-built-using-cuda-toolkit-11-8", "display_name": "Applications Built Using CUDA Toolkit 11.8", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "applications-built-using-cuda-toolkit-11-8", "priority": -1, "content": "CUDA applications built using CUDA Toolkit 11.8 are compatible with Hopper architecture as long as they are built to include kernels in native cubin (compute capability 9.0) or PTX form or both.", "keywords": []}, {"id": 5, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#building-applications-using-cuda-toolkit-11-7-or-earlier", "display_name": "Building Applications Using CUDA Toolkit 11.7 or Earlier", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "building-applications-using-cuda-toolkit-11-7-or-earlier", "priority": -1, "content": "The nvcc compiler included with version 11.7 or earlier (11.0-11.7) of the CUDA Toolkit can generate cubins native to the NVIDIA Ampere GPU architectures (compute capability 8.x). When using CUDA Toolkit 11.7 or earlier, to ensure that nvcc will generate cubin files for all recent GPU architectures as well as a PTX version for forward compatibility with future GPU architectures, specify the appropriate -gencode= parameters on the nvcc command line as shown in the examples below. Windows nvcc.exe -ccbin &quot;C:\\vs2010\\VC\\bin&quot; -Xcompiler &quot;/EHsc /W3 /nologo /O2 /Zi /MT&quot; -gencode=arch=compute_52,code=sm_52 -gencode=arch=compute_60,code=sm_60 -gencode=arch=compute_61,code=sm_61 -gencode=arch=compute_70,code=sm_70 -gencode=arch=compute_75,code=sm_75 -gencode=arch=compute_80,code=sm_80 -gencode=arch=compute_80,code=compute_80 --compile -o &quot;Release\\mykernel.cu.obj&quot; &quot;mykernel.cu&quot; Mac/Linux /usr/local/cuda/bin/nvcc -gencode=arch=compute_52,code=sm_52 -gencode=arch=compute_60,code=sm_60 -gencode=arch=compute_61,code=sm_61 -gencode=arch=compute_70,code=sm_70 -gencode=arch=compute_75,code=sm_75 -gencode=arch=compute_80,code=sm_80 -gencode=arch=compute_80,code=compute_80 -O2 -o mykernel.o -c mykernel.cu Alternatively, the simplified nvcc command-line option -arch=sm_XX can be used. It is a shorthand equivalent to the following more explicit -gencode= command-line options used above. -arch=sm_XX expands to the following: -gencode=arch=compute_XX,code=sm_XX -gencode=arch=compute_XX,code=compute_XX However, while the -arch=sm_XX command-line option does result in inclusion of a PTX back-end target binary by default, it can only specify a single target cubin architecture at a time, and it is not possible to use multiple -arch= options on the same nvcc command line, which is why the examples above use -gencode= explicitly. For CUDA toolkits prior to 11.0, one or more of the -gencode options need to be removed according to the architectures supported by the specific toolkit version (for example, CUDA toolkit 10.x supports architectures up to sm_72 and sm_75). The final -gencode to generate PTX also needs to be updated. For further information and examples see the documentation for the specific CUDA toolkit version. compute_XX refers to a PTX version and sm_XX refers to a cubin version. The arch= clause of the -gencode= command-line option to nvcc specifies the front-end compilation target and must always be a PTX version. The code= clause specifies the back-end compilation target and can either be cubin or PTX or both. Only the back-end target version(s) specified by the code= clause will be retained in the resulting binary; at least one should be PTX to provide compatibility with future architectures.", "keywords": []}, {"id": 6, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#building-applications-using-cuda-toolkit-11-8", "display_name": "Building Applications Using CUDA Toolkit 11.8", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "building-applications-using-cuda-toolkit-11-8", "priority": -1, "content": "With versions 11.8 of the CUDA Toolkit, nvcc can generate cubin native to the Hopper architecture (compute capability 9.0). When using CUDA Toolkit 11.8, to ensure that nvcc will generate cubin files for all recent GPU architectures as well as a PTX version for forward compatibility with future GPU architectures, specify the appropriate -gencode= parameters on the nvcc command line as shown in the examples below. Windows nvcc.exe -ccbin &quot;C:\\vs2010\\VC\\bin&quot; -Xcompiler &quot;/EHsc /W3 /nologo /O2 /Zi /MT&quot; -gencode=arch=compute_52,code=sm_52 -gencode=arch=compute_60,code=sm_60 -gencode=arch=compute_61,code=sm_61 -gencode=arch=compute_70,code=sm_70 -gencode=arch=compute_75,code=sm_75 -gencode=arch=compute_75,code=sm_75 -gencode=arch=compute_90,code=sm_90 -gencode=arch=compute_90,code=compute_90 --compile -o &quot;Release\\mykernel.cu.obj&quot; &quot;mykernel.cu&quot; Mac/Linux /usr/local/cuda/bin/nvcc -gencode=arch=compute_52,code=sm_52 -gencode=arch=compute_60,code=sm_60 -gencode=arch=compute_61,code=sm_61 -gencode=arch=compute_70,code=sm_70 -gencode=arch=compute_75,code=sm_75 -gencode=arch=compute_80,code=sm_80 -gencode=arch=compute_90,code=sm_90 -gencode=arch=compute_90,code=compute_90 -O2 -o mykernel.o -c mykernel.cu compute_XX refers to a PTX version and sm_XX refers to a cubin version. The arch= clause of the -gencode= command-line option to nvcc specifies the front-end compilation target and must always be a PTX version. The code= clause specifies the back-end compilation target and can either be cubin or PTX or both. Only the back-end target version(s) specified by the code= clause will be retained in the resulting binary; at least one should be PTX to provide compatibility with future architectures.", "keywords": []}, {"id": 7, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#building-applications-with-hopper-architecture-support", "display_name": "Building Applications with Hopper Architecture Support", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "building-applications-with-hopper-architecture-support", "priority": -1, "content": "Depending on the version of the CUDA Toolkit used for building the application, it can be built to include PTX and/or native cubin for the Hopper architecture. Although it is enough to just include PTX, including native cubin also has the following advantages: It saves the end user the time it takes to JIT-compile kernels that are available only as PTX. All kernels which do not have native cubins are JIT-compiled from PTX, including kernels from all the libraries linked to the application, even if those kernels are never launched by the application 2 . Especially when using large libraries, this JIT compilation can take a significant amount of time. The CUDA driver caches the cubins generated as a result of the PTX JIT, so this is mostly a one-time cost for a user, but it is time best avoided whenever possible. PTX JIT-compiled kernels often cannot take advantage of architectural features of newer GPUs, meaning that native-compiled cubins may be faster or of greater accuracy.", "keywords": []}, {"id": 8, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#independent-thread-scheduling-compatibility", "display_name": "Independent Thread Scheduling Compatibility", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "independent-thread-scheduling-compatibility", "priority": -1, "content": "NVIDIA GPUs since Volta architecture have Independent Thread Scheduling among threads in a warp. If the developer made assumptions about warp-synchronicity 3 , this feature can alter the set of threads participating in the executed code compared to previous architectures. Please see Compute Capability 7.x in the CUDA C++ Programming Guide for details and corrective actions. To aid migration to the Hopper architecture, developers can opt-in to the Pascal scheduling model with the following combination of compiler options. nvcc -gencode=arch=compute_60,code=sm_90 ...", "keywords": []}, {"id": 9, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#notice", "display_name": "Notice", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "notice", "priority": -1, "content": "This document is provided for information purposes only and shall not be regarded as a warranty of a certain functionality, condition, or quality of a product. NVIDIA Corporation (\u201cNVIDIA\u201d) makes no representations or warranties, expressed or implied, as to the accuracy or completeness of the information contained in this document and assumes no responsibility for any errors contained herein. NVIDIA shall have no liability for the consequences or use of such information or for any infringement of patents or other rights of third parties that may result from its use. This document is not a commitment to develop, release, or deliver any Material (defined below), code, or functionality. NVIDIA reserves the right to make corrections, modifications, enhancements, improvements, and any other changes to this document, at any time without notice. Customer should obtain the latest relevant information before placing orders and should verify that such information is current and complete. NVIDIA products are sold subject to the NVIDIA standard terms and conditions of sale supplied at the time of order acknowledgement, unless otherwise agreed in an individual sales agreement signed by authorized representatives of NVIDIA and customer (\u201cTerms of Sale\u201d). NVIDIA hereby expressly objects to applying any customer general terms and conditions with regards to the purchase of the NVIDIA product referenced in this document. No contractual obligations are formed either directly or indirectly by this document. NVIDIA products are not designed, authorized, or warranted to be suitable for use in medical, military, aircraft, space, or life support equipment, nor in applications where failure or malfunction of the NVIDIA product can reasonably be expected to result in personal injury, death, or property or environmental damage. NVIDIA accepts no liability for inclusion and/or use of NVIDIA products in such equipment or applications and therefore such inclusion and/or use is at customer\u2019s own risk. NVIDIA makes no representation or warranty that products based on this document will be suitable for any specified use. Testing of all parameters of each product is not necessarily performed by NVIDIA. It is customer\u2019s sole responsibility to evaluate and determine the applicability of any information contained in this document, ensure the product is suitable and fit for the application planned by customer, and perform the necessary testing for the application in order to avoid a default of the application or the product. Weaknesses in customer\u2019s product designs may affect the quality and reliability of the NVIDIA product and may result in additional or different conditions and/or requirements beyond those contained in this document. NVIDIA accepts no liability related to any default, damage, costs, or problem which may be based on or attributable to: (i) the use of the NVIDIA product in any manner that is contrary to this document or (ii) customer product designs. No license, either expressed or implied, is granted under any NVIDIA patent right, copyright, or other NVIDIA intellectual property right under this document. Information published by NVIDIA regarding third-party products or services does not constitute a license from NVIDIA to use such products or services or a warranty or endorsement thereof. Use of such information may require a license from a third party under the patents or other intellectual property rights of the third party, or a license from NVIDIA under the patents or other intellectual property rights of NVIDIA. Reproduction of information in this document is permissible only if approved in advance by NVIDIA in writing, reproduced without alteration and in full compliance with all applicable export laws and regulations, and accompanied by all associated conditions, limitations, and notices. THIS DOCUMENT AND ALL NVIDIA DESIGN SPECIFICATIONS, REFERENCE BOARDS, FILES, DRAWINGS, DIAGNOSTICS, LISTS, AND OTHER DOCUMENTS (TOGETHER AND SEPARATELY, \u201cMATERIALS\u201d) ARE BEING PROVIDED \u201cAS IS.\u201d NVIDIA MAKES NO WARRANTIES, EXPRESSED, IMPLIED, STATUTORY, OR OTHERWISE WITH RESPECT TO THE MATERIALS, AND EXPRESSLY DISCLAIMS ALL IMPLIED WARRANTIES OF NONINFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE. TO THE EXTENT NOT PROHIBITED BY LAW, IN NO EVENT WILL NVIDIA BE LIABLE FOR ANY DAMAGES, INCLUDING WITHOUT LIMITATION ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL, PUNITIVE, OR CONSEQUENTIAL DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF ANY USE OF THIS DOCUMENT, EVEN IF NVIDIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. Notwithstanding any damages that customer might incur for any reason whatsoever, NVIDIA\u2019s aggregate and cumulative liability towards customer for the products described herein shall be limited in accordance with the Terms of Sale for the product. LegacyDitaAnchor opencl", "keywords": []}, {"id": 10, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#notices", "display_name": "Notices", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "notices", "priority": -1, "content": "", "keywords": []}, {"id": 11, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#opencl", "display_name": "OpenCL", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "opencl", "priority": -1, "content": "OpenCL is a trademark of Apple Inc. used under license to the Khronos Group Inc. LegacyDitaAnchor trademarks", "keywords": []}, {"id": 12, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#revision-history", "display_name": "Revision History", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "revision-history", "priority": -1, "content": "Version 1.0 Initial public release. LegacyDitaAnchor notice", "keywords": []}, {"id": 13, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#trademarks", "display_name": "Trademarks", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "trademarks", "priority": -1, "content": "NVIDIA and the NVIDIA logo are trademarks or registered trademarks of NVIDIA Corporation in the U.S. and other countries. Other company and product names may be trademarks of the respective companies with which they are associated. 1 Just-in-time compilation. 2 Starting with CUDA toolkit 11.8, this default behavior can be changed with environment variable CUDA_MODULE_LOADING. See Environment Variables in the CUDA C++ Programming Guide for details. 3 Warp-synchronous refers to an assumption that threads in the same warp are synchronized at every instruction and can, for example, communicate values without explicit synchronization.", "keywords": []}, {"id": 14, "doc_id": 15, "filename": "index.html", "domain_name": "page", "name": "index#verifying-hopper-compatibility-for-existing-applications", "display_name": "Verifying Hopper Compatibility for Existing Applications", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "verifying-hopper-compatibility-for-existing-applications", "priority": -1, "content": "The first step towards making a CUDA application compatible with Hopper architecture is to check if the application binary already contains compatible GPU code (at least the PTX). The following sections explain how to accomplish this for an already built CUDA application.", "keywords": []}, {"id": 15, "doc_id": 15, "filename": "index.html", "domain_name": "std", "name": "index", "display_name": "Hopper Architecture Compatibility", "type": "doc", "display_type": "Page", "docname": "index", "anchor": "", "priority": -1, "content": "Hopper Compatibility Guide for CUDA Applications The guide to building CUDA applications for Hopper GPUs", "keywords": []}]};