const searchData = {"envversion": {"sphinx.domains.c": 2, "sphinx.domains.changeset": 1, "sphinx.domains.citation": 1, "sphinx.domains.cpp": 5, "sphinx.domains.index": 1, "sphinx.domains.javascript": 2, "sphinx.domains.math": 2, "sphinx.domains.python": 3, "sphinx.domains.rst": 2, "sphinx.domains.std": 2, "sphinx.ext.intersphinx": 1, "repo_docs.ext.enhanced_search": 1, "sphinx": 56}, "data": [{"id": 0, "doc_id": 0, "filename": "contents.html", "domain_name": "std", "name": "contents", "display_name": "Contents", "type": "doc", "display_type": "Page", "docname": "contents", "anchor": "", "priority": -1, "content": "NVIDIA Hopper Tuning Guide NVIDIA Hopper GPU Architecture CUDA Best Practices Application Compatibility NVIDIA Hopper Tuning Streaming Multiprocessor Occupancy Tensor Memory Accelerator Thread Block Clusters Improved FP32 Throughput Dynamic Programming Instructions Memory System High-Bandwidth Memory HBM3 Subsystem Increased L2 Capacity Inline Compression Unified Shared Memory/L1/Texture Cache Fourth-Generation NVLink Revision History Notices Notice OpenCL Trademarks", "keywords": []}, {"id": 1, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#application-compatibility", "display_name": "Application Compatibility", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "application-compatibility", "priority": -1, "content": "Before addressing specific performance tuning issues covered in this guide, refer to the Hopper Compatibility Guide for CUDA Applications to ensure that your application is compiled in a way that is compatible with NVIDIA Hopper.", "keywords": []}, {"id": 2, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#cuda-best-practices", "display_name": "CUDA Best Practices", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "cuda-best-practices", "priority": -1, "content": "The performance guidelines and best practices described in the CUDA C++ Programming Guide and the CUDA C++ Best Practices Guide apply to all CUDA-capable GPU architectures. Programmers must primarily focus on following those recommendations to achieve the best performance. The high-priority recommendations from those guides are as follows: Find ways to parallelize sequential code. Minimize data transfers between the host and the device. Adjust kernel launch configuration to maximize device utilization. Ensure that global memory accesses are coalesced. Minimize redundant accesses to global memory whenever possible. Avoid long sequences of diverged execution by threads within the same warp.", "keywords": []}, {"id": 3, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#dynamic-programming-instructions", "display_name": "Dynamic Programming Instructions", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "dynamic-programming-instructions", "priority": -1, "content": "The NVIDIA Hopper architecture adds support for new instructions to accelerate dynamic programming algorithms, such as the Smith-Waterman algorithm for sequence alignment in bioinformatics, and algorithms in graph theory, game theory, ML, and finance problems. The new instructions permit computation of max and min values among three operands, max and min operations yielding predicates, combined add operation with max or min, operating on signed and unsigned 32-bit int and 16-bit short2 types, and half2. All DPX instructions with 16-bit short types DPX instructions enable 128 operations per cycle per SM.", "keywords": []}, {"id": 4, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#fourth-generation-nvlink", "display_name": "Fourth-Generation NVLink", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "fourth-generation-nvlink", "priority": -1, "content": "The fourth generation of NVIDIA\u2019s high-speed NVLink interconnect is implemented in H100 GPUs, which significantly enhances multi-GPU scalability, performance, and reliability with more links per GPU, much faster communication bandwidth, and improved error-detection and recovery features. The fourth-generation NVLink has the same bidirectional data rate of 50 GB/s per link. The total number of links available is increased to 18 in H100, compared to 12 in A100, yielding 900 GB/s bidirectional bandwidth compared to 600 GB/s for A100. NVLink operates transparently within the existing CUDA model. Transfers between NVLink-connected endpoints are automatically routed through NVLink, rather than PCIe. The cudaDeviceEnablePeerAccess() API call remains necessary to enable direct transfers (over either PCIe or NVLink) between GPUs. The cudaDeviceCanAccessPeer() can be used to determine if peer access is possible between any pair of GPUs.", "keywords": []}, {"id": 5, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#high-bandwidth-memory-hbm3-subsystem", "display_name": "High-Bandwidth Memory HBM3 Subsystem", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "high-bandwidth-memory-hbm3-subsystem", "priority": -1, "content": "The NVIDIA H100 GPU has support for HBM3 and HBM2e memory, with capacity up to 80 GB. GPUs HBM3 memory system supports up to 3 TB/s memory bandwidth, a 93% increase over the 1.55 TB/s on A100-40GB.", "keywords": []}, {"id": 6, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#improved-fp32-throughput", "display_name": "Improved FP32 Throughput", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "improved-fp32-throughput", "priority": -1, "content": "Devices of compute capability 9.0 have 2x more FP32 operations per cycle per SM than devices of compute capability 8.0.", "keywords": []}, {"id": 7, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#increased-l2-capacity", "display_name": "Increased L2 Capacity", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "increased-l2-capacity", "priority": -1, "content": "The NVIDIA Hopper architecture increases the L2 cache capacity from 40 MB in the A100 GPU to 50 MB in the H100 GPU. Along with the increased capacity, the bandwidth of the L2 cache to the SMs is also increased. The NVIDIA Hopper architecture allows CUDA users to control the persistence of data in L2 cache similar to the NVIDIA Ampere GPU Architecture. For more information on the persistence of data in L2 cache, refer to the section on managing L2 cache in the CUDA C++ Programming Guide .", "keywords": []}, {"id": 8, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#inline-compression", "display_name": "Inline Compression", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "inline-compression", "priority": -1, "content": "The NVIDIA Hopper architecture allows CUDA compute kernels to benefit from the new inline compression (ILC). This feature can be applied to individual memory allocation, and the compressor automatically chooses between several possible compression algorithms, or none if there is no suitable pattern. In case compression can be used, this feature allows accessing global memory at significantly higher bandwidth than global memory bandwidth, since only compressed data needs to be transferred between global memory and SMs. However, the feature does not allow for reducing memory footprint: since compression is automatic, even if compression is active, the memory region will use the same footprint as if there was no compression. This is because underlying data may be changed by the user application and may not be compressible during the entire duration of the application. The feature is available through the CUDA driver API. See the CUDA C++ Programming Guide section on compressible memory : CUmemGenericAllocationHandle allocationHandle; CUmemAllocationProp prop = {}; memset(prop, 0, sizeof(CUmemAllocationProp)); prop-&gt;type = CU_MEM_ALLOCATION_TYPE_PINNED; prop-&gt;location.type = CU_MEM_LOCATION_TYPE_DEVICE; prop-&gt;location.id = currentDevice; prop-&gt;allocFlags.compressionType = CU_MEM_ALLOCATION_COMP_GENERIC; cuMemCreate(&amp;allocationHandle, size, &amp;prop, 0); One can check whether compressible memory is available on the given device with: cuDeviceGetAttribute(&amp;compressionAvailable, CU_DEVICE_ATTRIBUTE_GENERIC_COMPRESSION_SUPPORTED, currentDevice) Note that this example code does not handle errors and compiling this code requires linking against the CUDA library ( libcuda.so ).", "keywords": []}, {"id": 9, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#memory-system", "display_name": "Memory System", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "memory-system", "priority": -1, "content": "", "keywords": []}, {"id": 10, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#notice", "display_name": "Notice", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "notice", "priority": -1, "content": "This document is provided for information purposes only and shall not be regarded as a warranty of a certain functionality, condition, or quality of a product. NVIDIA Corporation (\u201cNVIDIA\u201d) makes no representations or warranties, expressed or implied, as to the accuracy or completeness of the information contained in this document and assumes no responsibility for any errors contained herein. NVIDIA shall have no liability for the consequences or use of such information or for any infringement of patents or other rights of third parties that may result from its use. This document is not a commitment to develop, release, or deliver any Material (defined below), code, or functionality. NVIDIA reserves the right to make corrections, modifications, enhancements, improvements, and any other changes to this document, at any time without notice. Customer should obtain the latest relevant information before placing orders and should verify that such information is current and complete. NVIDIA products are sold subject to the NVIDIA standard terms and conditions of sale supplied at the time of order acknowledgement, unless otherwise agreed in an individual sales agreement signed by authorized representatives of NVIDIA and customer (\u201cTerms of Sale\u201d). NVIDIA hereby expressly objects to applying any customer general terms and conditions with regards to the purchase of the NVIDIA product referenced in this document. No contractual obligations are formed either directly or indirectly by this document. NVIDIA products are not designed, authorized, or warranted to be suitable for use in medical, military, aircraft, space, or life support equipment, nor in applications where failure or malfunction of the NVIDIA product can reasonably be expected to result in personal injury, death, or property or environmental damage. NVIDIA accepts no liability for inclusion and/or use of NVIDIA products in such equipment or applications and therefore such inclusion and/or use is at customer\u2019s own risk. NVIDIA makes no representation or warranty that products based on this document will be suitable for any specified use. Testing of all parameters of each product is not necessarily performed by NVIDIA. It is customer\u2019s sole responsibility to evaluate and determine the applicability of any information contained in this document, ensure the product is suitable and fit for the application planned by customer, and perform the necessary testing for the application in order to avoid a default of the application or the product. Weaknesses in customer\u2019s product designs may affect the quality and reliability of the NVIDIA product and may result in additional or different conditions and/or requirements beyond those contained in this document. NVIDIA accepts no liability related to any default, damage, costs, or problem which may be based on or attributable to: (i) the use of the NVIDIA product in any manner that is contrary to this document or (ii) customer product designs. No license, either expressed or implied, is granted under any NVIDIA patent right, copyright, or other NVIDIA intellectual property right under this document. Information published by NVIDIA regarding third-party products or services does not constitute a license from NVIDIA to use such products or services or a warranty or endorsement thereof. Use of such information may require a license from a third party under the patents or other intellectual property rights of the third party, or a license from NVIDIA under the patents or other intellectual property rights of NVIDIA. Reproduction of information in this document is permissible only if approved in advance by NVIDIA in writing, reproduced without alteration and in full compliance with all applicable export laws and regulations, and accompanied by all associated conditions, limitations, and notices. THIS DOCUMENT AND ALL NVIDIA DESIGN SPECIFICATIONS, REFERENCE BOARDS, FILES, DRAWINGS, DIAGNOSTICS, LISTS, AND OTHER DOCUMENTS (TOGETHER AND SEPARATELY, \u201cMATERIALS\u201d) ARE BEING PROVIDED \u201cAS IS.\u201d NVIDIA MAKES NO WARRANTIES, EXPRESSED, IMPLIED, STATUTORY, OR OTHERWISE WITH RESPECT TO THE MATERIALS, AND EXPRESSLY DISCLAIMS ALL IMPLIED WARRANTIES OF NONINFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE. TO THE EXTENT NOT PROHIBITED BY LAW, IN NO EVENT WILL NVIDIA BE LIABLE FOR ANY DAMAGES, INCLUDING WITHOUT LIMITATION ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL, PUNITIVE, OR CONSEQUENTIAL DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF ANY USE OF THIS DOCUMENT, EVEN IF NVIDIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. Notwithstanding any damages that customer might incur for any reason whatsoever, NVIDIA\u2019s aggregate and cumulative liability towards customer for the products described herein shall be limited in accordance with the Terms of Sale for the product. LegacyDitaAnchor opencl", "keywords": []}, {"id": 11, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#notices", "display_name": "Notices", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "notices", "priority": -1, "content": "", "keywords": []}, {"id": 12, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#nvidia-hopper-gpu-architecture", "display_name": "NVIDIA Hopper GPU Architecture", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "nvidia-hopper-gpu-architecture", "priority": -1, "content": "The NVIDIA\u00ae Hopper GPU architecture is NVIDIA\u2019s latest architecture for CUDA\u00ae compute applications. The NVIDIA Hopper GPU architecture retains and extends the same CUDA programming model provided by previous NVIDIA GPU architectures such as NVIDIA Ampere GPU architecture and NVIDIA Turing, and applications that follow the best practices for those architectures should typically see speedups on the NVIDIA H100 GPU without any code changes. This guide summarizes the ways that an application can be fine-tuned to gain additional speedups by leveraging the NVIDIA Hopper GPU architecture\u2019s features. 1 For further details on the programming features discussed in this guide, refer to the CUDA C++ Programming Guide .", "keywords": []}, {"id": 13, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#nvidia-hopper-tuning", "display_name": "NVIDIA Hopper Tuning", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "nvidia-hopper-tuning", "priority": -1, "content": "", "keywords": []}, {"id": 14, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#occupancy", "display_name": "Occupancy", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "occupancy", "priority": -1, "content": "The maximum number of concurrent warps per SM remains the same as in NVIDIA Ampere GPU architecture (that is, 64), and other factors influencing warp occupancy are: The register file size is 64K 32-bit registers per SM. The maximum number of registers per thread is 255. The maximum number of thread blocks per SM is 32 for devices of compute capability 9.0 (that is, H100 GPUs). For devices of compute capability 9.0 (H100 GPUs), shared memory capacity per SM is 228 KB, a 39% increase compared to A100\u2019s capacity of 164 KB. For devices of compute capability 9.0 (H100 GPUs), the maximum shared memory per thread block is 227 KB. For applications using Thread Block Clusters, it is always recommended to compute the occupancy using cudaOccupancyMaxActiveClusters and launch cluster-based kernels accordingly. Overall, developers can expect similar occupancy as on NVIDIA Ampere GPU architecture GPUs without changes to their application.", "keywords": []}, {"id": 15, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#opencl", "display_name": "OpenCL", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "opencl", "priority": -1, "content": "OpenCL is a trademark of Apple Inc. used under license to the Khronos Group Inc. LegacyDitaAnchor trademarks", "keywords": []}, {"id": 16, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#revision-history", "display_name": "Revision History", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "revision-history", "priority": -1, "content": "Version 1.0 Initial Public Release Added support for compute capability 9.0 1 Throughout this guide, NVIDIA Volta refers to devices of compute capability 7.0, NVIDIA Turing refers to devices of compute capability 7.5, NVIDIA Ampere GPU Architecture refers to devices of compute capability 8.x, and NVIDIA Hopper refers to devices of compute capability 9.0. LegacyDitaAnchor notice", "keywords": []}, {"id": 17, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#streaming-multiprocessor", "display_name": "Streaming Multiprocessor", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "streaming-multiprocessor", "priority": -1, "content": "The NVIDIA Hopper Streaming Multiprocessor (SM) provides the following improvements over Turing and NVIDIA Ampere GPU architectures.", "keywords": []}, {"id": 18, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#tensor-memory-accelerator", "display_name": "Tensor Memory Accelerator", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "tensor-memory-accelerator", "priority": -1, "content": "The Hopper architecture builds on top of the asynchronous copies introduced by NVIDIA Ampere GPU architecture and provides a more sophisticated asynchronous copy engine: the Tensor Memory Accelerator (TMA). TMA allows applications to transfer 1D and up to 5D tensors between global memory and shared memory, in both directions, as well as between the shared memory regions of different SMs in the same cluster (refer to Thread Block Clusters ). Additionally, for writes from shared memory to global memory, it allows specifying element wise reduction operations such as add/min/max as well as bitwise and/or for most common data types. This has several advantages: Avoids using registers for moving data between the different memory spaces. Avoids using SM instructions for moving data: a single thread can issue large data movement instructions to the TMA unit. The whole block can then continue working on other instructions while the data is in flight and only wait for the data to be consumed when actually necessary. Enables users to write warp specialized codes, where specific warps specialize on data movement between the different memory spaces while other warps only work on local data within the SM. This feature will be exposed through cuda::memcpy_async along with the cuda::barrier and cuda::pipeline for synchronizing data movement.", "keywords": []}, {"id": 19, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#thread-block-clusters", "display_name": "Thread Block Clusters", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "thread-block-clusters", "priority": -1, "content": "NVIDIA Hopper Architecture adds a new optional level of hierarchy, Thread Block Clusters, that allows for further possibilities when parallelizing applications. A thread block can read from, write to, and perform atomics in shared memory of other thread blocks within its cluster. This is known as Distributed Shared Memory. As demonstrated in the CUDA C++ Programming Guide , there are applications that cannot fit required data within shared memory and must use global memory instead. Distributed shared memory can act as an intermediate step between these two options. Distributed Shared Memory can be used by an SM simultaneously with L2 cache accesses. This can benefit applications that need to communicate data between SMs by utilizing the combined bandwidth of both distributed shared memory and L2. The maximum portable cluster size supported is 8; however, NVIDIA Hopper H100 GPU allows for a nonportable cluster size of 16 by opting in. Launching a kernel with a nonportable cluster size requires setting the cudaFuncAttributeNonPortableClusterSizeAllowed function attribute. Using larger cluster sizes may reduce the maximum number of active blocks across the GPU (refer to Occupancy ).", "keywords": []}, {"id": 20, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#trademarks", "display_name": "Trademarks", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "trademarks", "priority": -1, "content": "NVIDIA and the NVIDIA logo are trademarks or registered trademarks of NVIDIA Corporation in the U.S. and other countries. Other company and product names may be trademarks of the respective companies with which they are associated.", "keywords": []}, {"id": 21, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#unified-shared-memory-l1-texture-cache", "display_name": "Unified Shared Memory/L1/Texture Cache", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "unified-shared-memory-l1-texture-cache", "priority": -1, "content": "The NVIDIA H100 GPU based on compute capability 9.0 increases the maximum capacity of the combined L1 cache, texture cache, and shared memory to 256 KB, from 192 KB in NVIDIA Ampere Architecture, an increase of 33%. In the NVIDIA Hopper GPU architecture, the portion of the L1 cache dedicated to shared memory (known as the carveout) can be selected at runtime as in previous architectures such as NVIDIA Ampere Architecture and NVIDIA Volta, using cudaFuncSetAttribute() with the attribute cudaFuncAttributePreferredSharedMemoryCarveout . The NVIDIA H100 GPU supports shared memory capacities of 0, 8, 16, 32, 64, 100, 132, 164, 196 and 228 KB per SM. CUDA reserves 1 KB of shared memory per thread block. Hence, the H100 GPU enables a single thread block to address up to 227 KB of shared memory. To maintain architectural compatibility, static shared memory allocations remain limited to 48 KB, and an explicit opt-in is also required to enable dynamic allocations above this limit. See the CUDA C++ Programming Guide for details. Like the NVIDIA Ampere Architecture and NVIDIA Volta GPU architectures, the NVIDIA Hopper GPU architecture combines the functionality of the L1 and texture caches into a unified L1/Texture cache which acts as a coalescing buffer for memory accesses, gathering up the data requested by the threads of a warp before delivery of that data to the warp. Another benefit of its union with shared memory, similar to previous architectures, is improvement in terms of both latency and bandwidth.", "keywords": []}, {"id": 22, "doc_id": 22, "filename": "index.html", "domain_name": "std", "name": "index", "display_name": "NVIDIA Hopper Tuning Guide", "type": "doc", "display_type": "Page", "docname": "index", "anchor": "", "priority": -1, "content": "Tuning CUDA Applications for Hopper GPU Architecture The programming guide for tuning CUDA Applications for GPUs based on the Hopper GPU Architecture.", "keywords": []}]};