const searchData = {"envversion": {"sphinx.domains.c": 2, "sphinx.domains.changeset": 1, "sphinx.domains.citation": 1, "sphinx.domains.cpp": 5, "sphinx.domains.index": 1, "sphinx.domains.javascript": 2, "sphinx.domains.math": 2, "sphinx.domains.python": 3, "sphinx.domains.rst": 2, "sphinx.domains.std": 2, "sphinx.ext.intersphinx": 1, "repo_docs.ext.enhanced_search": 1, "sphinx": 56}, "data": [{"id": 0, "doc_id": 0, "filename": "contents.html", "domain_name": "std", "name": "contents", "display_name": "Contents", "type": "doc", "display_type": "Page", "docname": "contents", "anchor": "", "priority": -1, "content": "Introduction Installation and Versioning Vectors Thrust Namespace Iterators and Static Dispatching Algorithms Transformations Reductions Prefix-Sums Reordering Sorting Fancy Iterators constant_iterator counting_iterator transform_iterator permutation_iterator zip_iterator Additional Resources Notices Notice OpenCL Trademarks", "keywords": []}, {"id": 1, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#additional-resources", "display_name": "Additional Resources", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "additional-resources", "priority": -1, "content": "This guide only scratches the surface of what you can do with Thrust. The following resources can help you learn to do more with Thrust or provide assistance when problems arise. Comprehensive documentation of Thrust\u2019s API A list of Frequently Asked Questions Collection of example programs We strongly encourage users to subscribe to the thrust-users mailing list. The mailing list is a great place to seek out help from the Thrust developers and other Thrust users. LegacyDitaAnchor notice", "keywords": []}, {"id": 2, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#algorithms", "display_name": "Algorithms", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "algorithms", "priority": -1, "content": "Thrust provides a large number of common parallel algorithms. Many of these algorithms have direct analogs in the STL, and when an equivalent STL function exists, we choose the name (e.g. thrust::sort and std::sort ). All algorithms in Thrust have implementations for both host and device. Specifically, when a Thrust algorithm is invoked with a host iterator, then the host path is dispatched. Similarly, a device implementation is called when a device iterator is used to define a range. With the exception of thrust::copy , which can copy data between host and device, all iterator arguments to a Thrust algorithm should live in the same place: either all on the host or all on the device. When this requirement is violated the compiler will produce an error message. LegacyDitaAnchor transformations", "keywords": []}, {"id": 3, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#constant-iterator", "display_name": "constant_iterator", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "constant-iterator", "priority": -1, "content": "Arguably the simplest of the bunch, constant_iterator is simply an iterator that returns the same value whenever we access it. In the following example we initialize a constant iterator with the value 10. #include &lt;thrust/iterator/constant_iterator.h&gt; ... // create iterators thrust::constant_iterator&lt;int&gt; first(10); thrust::constant_iterator&lt;int&gt; last = first + 3; first[0] // returns 10 first[1] // returns 10 first[100] // returns 10 // sum of [first, last) thrust::reduce(first, last); // returns 30 (i.e. 3 * 10) Whenever an input sequence of constant values is needed, constant_iterator is a convenient and efficient solution. LegacyDitaAnchor counting-iterator", "keywords": []}, {"id": 4, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#counting-iterator", "display_name": "counting_iterator", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "counting-iterator", "priority": -1, "content": "If a sequence of increasing values is required, then counting_iterator is the appropriate choice. Here we initialize a counting_iterator with the value 10 and access it like an array. #include &lt;thrust/iterator/counting_iterator.h&gt; ... // create iterators thrust::counting_iterator&lt;int&gt; first(10); thrust::counting_iterator&lt;int&gt; last = first + 3; first[0] // returns 10 first[1] // returns 11 first[100] // returns 110 // sum of [first, last) thrust::reduce(first, last); // returns 33 (i.e. 10 + 11 + 12) While constant_iterator and counting_iterator act as arrays, they don\u2019t actually require any memory storage. Whenever we dereference one of these iterators it generates the appropriate value on-the-fly and returns it to the calling function. LegacyDitaAnchor transform-iterator", "keywords": []}, {"id": 5, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#fancy-iterators", "display_name": "Fancy Iterators", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "fancy-iterators", "priority": -1, "content": "Fancy iterators perform a variety of valuable purposes. In this section we\u2019ll show how fancy iterators allow us to attack a broader class of problems with the standard Thrust algorithms. For those familiar with the Boost C++ Library , note that our fancy iterators were inspired by (and generally derived from) those in the Boost Iterator Library . LegacyDitaAnchor constant-iterator", "keywords": []}, {"id": 6, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#installation-and-versioning", "display_name": "Installation and Versioning", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "installation-and-versioning", "priority": -1, "content": "Installing the CUDA Toolkit will copy Thrust header files to the standard CUDA include directory for your system. Since Thrust is a template library of header files, no further installation is necessary to start using Thrust. In addition, new versions of Thrust continue to be available online through the GitHub Thrust project page . LegacyDitaAnchor vectors", "keywords": []}, {"id": 7, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#iterators-and-static-dispatching", "display_name": "Iterators and Static Dispatching", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "iterators-and-static-dispatching", "priority": -1, "content": "In this section we used expressions like H.begin() and H.end() or offsets like D.begin() + 7 . The result of begin() and end() is called an iterator in C++. In the case of vector containers, which are really just arrays, iterators can be thought of as pointers to array elements. Therefore, H.begin() is an iterator that points to the first element of the array stored inside the H vector. Similarly, H.end() points to the element one past the last element of the H vector. Although vector iterators are similar to pointers they carry more information with them. Notice that we did not have to tell thrust::fill that it was operating on a device_vector iterator. This information is captured in the type of the iterator returned by D.begin() which is different than the type returned by H.begin() . When a Thrust function is called, it inspects the type of the iterator to determine whether to use a host or a device implementation. This process is known as static dispatching since the host/device dispatch is resolved at compile time. Note that this implies that there is no runtime overhead to the dispatch process. You may wonder what happens when a \u201craw\u201d pointer is used as an argument to a Thrust function. Like the STL, Thrust permits this usage and it will dispatch the host path of the algorithm. If the pointer in question is in fact a pointer to device memory then you\u2019ll need to wrap it with thrust::device_ptr before calling the function. For example: size_t N = 10; // raw pointer to device memory int * raw_ptr; cudaMalloc((void **) &amp;raw_ptr, N * sizeof(int)); // wrap raw pointer with a device_ptr thrust::device_ptr&lt;int&gt; dev_ptr(raw_ptr); // use device_ptr in thrust algorithms thrust::fill(dev_ptr, dev_ptr + N, (int) 0); To extract a raw pointer from a device_ptr the raw_pointer_cast should be applied as follows: size_t N = 10; // create a device_ptr thrust::device_ptr&lt;int&gt; dev_ptr = thrust::device_malloc&lt;int&gt;(N); // extract raw pointer from device_ptr int * raw_ptr = thrust::raw_pointer_cast(dev_ptr); Another reason to distinguish between iterators and pointers is that iterators can be used to traverse many kinds of data structures. For example, the STL provides a linked list container ( std::list ) that provides bidirectional (but not random access) iterators. Although Thrust does not provide device implementations of such containers, it is compatible with them. #include &lt;thrust/device_vector.h&gt; #include &lt;thrust/copy.h&gt; #include &lt;list&gt; #include &lt;vector&gt; int main(void) { // create an STL list with 4 values std::list&lt;int&gt; stl_list; stl_list.push_back(10); stl_list.push_back(20); stl_list.push_back(30); stl_list.push_back(40); // initialize a device_vector with the list thrust::device_vector&lt;int&gt; D(stl_list.begin(), stl_list.end()); // copy a device_vector into an STL vector std::vector&lt;int&gt; stl_vector(D.size()); thrust::copy(D.begin(), D.end(), stl_vector.begin()); return 0; } For Future Reference: The iterators we\u2019ve covered so far are useful, but fairly basic. In addition to these normal iterators, Thrust also provides a collection of fancy iterators with names like counting_iterator and zip_iterator . While they look and feel like normal iterators, fancy iterators are capable of more exciting things. We\u2019ll revisit this topic later in the tutorial. LegacyDitaAnchor algorithms", "keywords": []}, {"id": 8, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#notice", "display_name": "Notice", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "notice", "priority": -1, "content": "This document is provided for information purposes only and shall not be regarded as a warranty of a certain functionality, condition, or quality of a product. NVIDIA Corporation (\u201cNVIDIA\u201d) makes no representations or warranties, expressed or implied, as to the accuracy or completeness of the information contained in this document and assumes no responsibility for any errors contained herein. NVIDIA shall have no liability for the consequences or use of such information or for any infringement of patents or other rights of third parties that may result from its use. This document is not a commitment to develop, release, or deliver any Material (defined below), code, or functionality. NVIDIA reserves the right to make corrections, modifications, enhancements, improvements, and any other changes to this document, at any time without notice. Customer should obtain the latest relevant information before placing orders and should verify that such information is current and complete. NVIDIA products are sold subject to the NVIDIA standard terms and conditions of sale supplied at the time of order acknowledgement, unless otherwise agreed in an individual sales agreement signed by authorized representatives of NVIDIA and customer (\u201cTerms of Sale\u201d). NVIDIA hereby expressly objects to applying any customer general terms and conditions with regards to the purchase of the NVIDIA product referenced in this document. No contractual obligations are formed either directly or indirectly by this document. NVIDIA products are not designed, authorized, or warranted to be suitable for use in medical, military, aircraft, space, or life support equipment, nor in applications where failure or malfunction of the NVIDIA product can reasonably be expected to result in personal injury, death, or property or environmental damage. NVIDIA accepts no liability for inclusion and/or use of NVIDIA products in such equipment or applications and therefore such inclusion and/or use is at customer\u2019s own risk. NVIDIA makes no representation or warranty that products based on this document will be suitable for any specified use. Testing of all parameters of each product is not necessarily performed by NVIDIA. It is customer\u2019s sole responsibility to evaluate and determine the applicability of any information contained in this document, ensure the product is suitable and fit for the application planned by customer, and perform the necessary testing for the application in order to avoid a default of the application or the product. Weaknesses in customer\u2019s product designs may affect the quality and reliability of the NVIDIA product and may result in additional or different conditions and/or requirements beyond those contained in this document. NVIDIA accepts no liability related to any default, damage, costs, or problem which may be based on or attributable to: (i) the use of the NVIDIA product in any manner that is contrary to this document or (ii) customer product designs. No license, either expressed or implied, is granted under any NVIDIA patent right, copyright, or other NVIDIA intellectual property right under this document. Information published by NVIDIA regarding third-party products or services does not constitute a license from NVIDIA to use such products or services or a warranty or endorsement thereof. Use of such information may require a license from a third party under the patents or other intellectual property rights of the third party, or a license from NVIDIA under the patents or other intellectual property rights of NVIDIA. Reproduction of information in this document is permissible only if approved in advance by NVIDIA in writing, reproduced without alteration and in full compliance with all applicable export laws and regulations, and accompanied by all associated conditions, limitations, and notices. THIS DOCUMENT AND ALL NVIDIA DESIGN SPECIFICATIONS, REFERENCE BOARDS, FILES, DRAWINGS, DIAGNOSTICS, LISTS, AND OTHER DOCUMENTS (TOGETHER AND SEPARATELY, \u201cMATERIALS\u201d) ARE BEING PROVIDED \u201cAS IS.\u201d NVIDIA MAKES NO WARRANTIES, EXPRESSED, IMPLIED, STATUTORY, OR OTHERWISE WITH RESPECT TO THE MATERIALS, AND EXPRESSLY DISCLAIMS ALL IMPLIED WARRANTIES OF NONINFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE. TO THE EXTENT NOT PROHIBITED BY LAW, IN NO EVENT WILL NVIDIA BE LIABLE FOR ANY DAMAGES, INCLUDING WITHOUT LIMITATION ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL, PUNITIVE, OR CONSEQUENTIAL DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF ANY USE OF THIS DOCUMENT, EVEN IF NVIDIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. Notwithstanding any damages that customer might incur for any reason whatsoever, NVIDIA\u2019s aggregate and cumulative liability towards customer for the products described herein shall be limited in accordance with the Terms of Sale for the product. LegacyDitaAnchor opencl", "keywords": []}, {"id": 9, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#notices", "display_name": "Notices", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "notices", "priority": -1, "content": "", "keywords": []}, {"id": 10, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#opencl", "display_name": "OpenCL", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "opencl", "priority": -1, "content": "OpenCL is a trademark of Apple Inc. used under license to the Khronos Group Inc. LegacyDitaAnchor trademarks", "keywords": []}, {"id": 11, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#permutation-iterator", "display_name": "permutation_iterator", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "permutation-iterator", "priority": -1, "content": "In the previous section we showed how transform_iterator is used to fuse a transformation with another algorithm to avoid unnecessary memory operations. The permutation_iterator is similar: it allows us to fuse gather and scatter operations with Thrust algorithms, or even other fancy iterators. The following example shows how to fuse a gather operation with a reduction. #include &lt;thrust/iterator/permutation_iterator.h&gt; ... // gather locations thrust::device_vector&lt;int&gt; map(4); map[0] = 3; map[1] = 1; map[2] = 0; map[3] = 5; // array to gather from thrust::device_vector&lt;int&gt; source(6); source[0] = 10; source[1] = 20; source[2] = 30; source[3] = 40; source[4] = 50; source[5] = 60; // fuse gather with reduction: // sum = source[map[0]] + source[map[1]] + ... int sum = thrust::reduce(thrust::make_permutation_iterator(source.begin(), map.begin()), thrust::make_permutation_iterator(source.begin(), map.end())); Here we have used the make_permutation_iterator function to simplify the construction of the permutation_iterator s. The first argument to make_permutation_iterator is the source array of the gather operation and the second is the list of map indices. Note that we pass in source.begin() for the first argument in both cases, but vary the second argument to define the beginning and end of the sequence. When a permutation_iterator is used as an output sequence of a function it is equivalent to fusing a scatter operation to the algorithm. In general permutation_iterator allows you to operate on a specific set of values in a sequence instead of the entire sequence. LegacyDitaAnchor zip-iterator", "keywords": []}, {"id": 12, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#prefix-sums", "display_name": "Prefix-Sums", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "prefix-sums", "priority": -1, "content": "Parallel prefix-sums, or scan operations, are important building blocks in many parallel algorithms such as stream compaction and radix sort. Consider the following source code which illustrates an inclusive scan operation using the default plus operator: #include &lt;thrust/scan.h&gt; int data[6] = {1, 0, 2, 2, 1, 3}; thrust::inclusive_scan(data, data + 6, data); // in-place scan // data is now {1, 1, 3, 5, 6, 9} In an inclusive scan each element of the output is the corresponding partial sum of the input range. For example, data[2] = data[0] + data[1] + data[2] . An exclusive scan is similar, but shifted by one place to the right: #include &lt;thrust/scan.h&gt; int data[6] = {1, 0, 2, 2, 1, 3}; thrust::exclusive_scan(data, data + 6, data); // in-place scan // data is now {0, 1, 1, 3, 5, 6} So now data[2] = data[0] + data[1] . As these examples show, inclusive_scan and exclusive_scan are permitted to be performed in-place. Thrust also provides the functions transform_inclusive_scan and transform_exclusive_scan which apply a unary function to the input sequence before performing the scan. Refer to the documentation for a complete list of scan variants. LegacyDitaAnchor reordering", "keywords": []}, {"id": 13, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#reductions", "display_name": "Reductions", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "reductions", "priority": -1, "content": "A reduction algorithm uses a binary operation to reduce an input sequence to a single value. For example, the sum of an array of numbers is obtained by reducing the array with a plus operation. Similarly, the maximum of an array is obtained by reducing with an operator that takes two inputs and returns the maximum. The sum of an array is implemented with thrust::reduce as follows: int sum = thrust::reduce(D.begin(), D.end(), (int) 0, thrust::plus&lt;int&gt;()); The first two arguments to reduce define the range of values while the third and fourth parameters provide the initial value and reduction operator respectively. Actually, this kind of reduction is so common that it is the default choice when no initial value or operator is provided. The following three lines are therefore equivalent: int sum = thrust::reduce(D.begin(), D.end(), (int) 0, thrust::plus&lt;int&gt;()); int sum = thrust::reduce(D.begin(), D.end(), (int) 0); int sum = thrust::reduce(D.begin(), D.end()) Although thrust::reduce is sufficient to implement a wide variety of reductions, Thrust provides a few additional functions for convenience (like the STL). For example, thrust::count returns the number of instances of a specific value in a given sequence: #include &lt;thrust/count.h&gt; #include &lt;thrust/device_vector.h&gt; ... // put three 1s in a device_vector thrust::device_vector&lt;int&gt; vec(5,0); vec[1] = 1; vec[3] = 1; vec[4] = 1; // count the 1s int result = thrust::count(vec.begin(), vec.end(), 1); // result is three Other reduction operations include thrust::count_if , thrust::min_element , thrust::max_element , thrust::is_sorted , thrust::inner_product , and several others. Refer to the documentation for a complete listing. The SAXPY example in the Transformations section showed how kernel fusion can be used to reduce the number of memory transfers used by a transformation kernel. With thrust::transform_reduce we can also apply kernel fusion to reduction kernels. Consider the following example which computes the norm of a vector. #include &lt;thrust/transform_reduce.h&gt; #include &lt;thrust/functional.h&gt; #include &lt;thrust/device_vector.h&gt; #include &lt;thrust/host_vector.h&gt; #include &lt;cmath&gt; // square&lt;T&gt; computes the square of a number f(x) -&gt; x*x template &lt;typename T&gt; struct square { __host__ __device__ T operator()(const T&amp; x) const { return x * x; } }; int main(void) { // initialize host array float x[4] = {1.0, 2.0, 3.0, 4.0}; // transfer to device thrust::device_vector&lt;float&gt; d_x(x, x + 4); // setup arguments square&lt;float&gt; unary_op; thrust::plus&lt;float&gt; binary_op; float init = 0; // compute norm float norm = std::sqrt( thrust::transform_reduce(d_x.begin(), d_x.end(), unary_op, init, binary_op) ); std::cout &lt;&lt; norm &lt;&lt; std::endl; return 0; } Here we have a unary operator called square that squares each element of the input sequence. The sum of squares is then computed using a standard plus reduction. Like the slower version of the SAXPY transformation, we could implement norm with multiple passes: first a transform using square or perhaps just multiplies and then a plus reduction over a temporary array. However this would be unnecessarily wasteful and considerably slower. By fusing the square operation with the reduction kernel we again have a highly optimized implementation which offers the same performance as hand-written kernels. LegacyDitaAnchor prefix-sums", "keywords": []}, {"id": 14, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#reordering", "display_name": "Reordering", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "reordering", "priority": -1, "content": "Thrust provides support for partitioning and stream compaction through the following algorithms: copy_if : copy elements that pass a predicate test partition : reorder elements according to a predicate (true values precede false values) remove and remove_if : remove elements that fail a predicate test unique : remove consecutive duplicates within a sequence Refer to the documentation for a complete list of reordering functions and examples of their usage. LegacyDitaAnchor sorting", "keywords": []}, {"id": 15, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#sorting", "display_name": "Sorting", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "sorting", "priority": -1, "content": "Thrust offers several functions to sort data or rearrange data according to a given criterion. The thrust::sort and thrust::stable_sort functions are direct analogs of sort and stable_sort in the STL. #include &lt;thrust/sort.h&gt; ... const int N = 6; int A[N] = {1, 4, 2, 8, 5, 7}; thrust::sort(A, A + N); // A is now {1, 2, 4, 5, 7, 8} In addition, Thrust provides thrust::sort_by_key and thrust::stable_sort_by_key , which sort key-value pairs stored in separate places. #include &lt;thrust/sort.h&gt; ... const int N = 6; int keys[N] = { 1, 4, 2, 8, 5, 7}; char values[N] = {&#x27;a&#x27;, &#x27;b&#x27;, &#x27;c&#x27;, &#x27;d&#x27;, &#x27;e&#x27;, &#x27;f&#x27;}; thrust::sort_by_key(keys, keys + N, values); // keys is now { 1, 2, 4, 5, 7, 8} // values is now {&#x27;a&#x27;, &#x27;c&#x27;, &#x27;b&#x27;, &#x27;e&#x27;, &#x27;f&#x27;, &#x27;d&#x27;} Like their STL brethren, the sorting functions also accept user-defined comparison operators: #include &lt;thrust/sort.h&gt; #include &lt;thrust/functional.h&gt; ... const int N = 6; int A[N] = {1, 4, 2, 8, 5, 7}; thrust::stable_sort(A, A + N, thrust::greater&lt;int&gt;()); // A is now {8, 7, 5, 4, 2, 1} LegacyDitaAnchor fancy-iterators", "keywords": []}, {"id": 16, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#thrust-namespace", "display_name": "Thrust Namespace", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "thrust-namespace", "priority": -1, "content": "You\u2019ll notice that we use things like thrust::host_vector or thrust::copy in our examples. The thrust:: part tells the C++ compiler that we want to look inside the thrust namespace for a specific function or class. Namespaces are a nice way to avoid name collisions. For instance, thrust::copy is different from std::copy provided in the STL. C++ namespaces allow us to distinguish between these two copy functions. LegacyDitaAnchor iterators-and-static-dispatching", "keywords": []}, {"id": 17, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#trademarks", "display_name": "Trademarks", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "trademarks", "priority": -1, "content": "NVIDIA and the NVIDIA logo are trademarks or registered trademarks of NVIDIA Corporation in the U.S. and other countries. Other company and product names may be trademarks of the respective companies with which they are associated.", "keywords": []}, {"id": 18, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#transform-iterator", "display_name": "transform_iterator", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "transform-iterator", "priority": -1, "content": "In the Algorithms section we spoke about kernel fusion, i.e. combining separate algorithms like transform and reduce into a single transform_reduce operation. The transform_iterator allows us to apply the same technique, even when we don\u2019t have a special transform_xxx version of the algorithm. This example shows another way to fuse a transformation with a reduction, this time with just plain reduce applied to a transform_iterator . #include &lt;thrust/iterator/transform_iterator.h&gt; // initialize vector thrust::device_vector&lt;int&gt; vec(3); vec[0] = 10; vec[1] = 20; vec[2] = 30; // create iterator (type omitted) ... first = thrust::make_transform_iterator(vec.begin(), negate&lt;int&gt;()); ... last = thrust::make_transform_iterator(vec.end(), negate&lt;int&gt;()); first[0] // returns -10 first[1] // returns -20 first[2] // returns -30 // sum of [first, last) thrust::reduce(first, last); // returns -60 (i.e. -10 + -20 + -30) Note, we have omitted the types for iterators first and last for simplicity. One downside of transform_iterator is that it can be cumbersome to specify the full type of the iterator, which can be quite lengthy. For this reason, it is common practice to simply put the call to make_transform_iterator in the arguments of the algorithm being invoked. For example, // sum of [first, last) thrust::reduce(thrust::make_transform_iterator(vec.begin(), negate&lt;int&gt;()), thrust::make_transform_iterator(vec.end(), negate&lt;int&gt;())); allows us to avoid creating a variable to store first and last . LegacyDitaAnchor permutation-iterator", "keywords": []}, {"id": 19, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#transformations", "display_name": "Transformations", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "transformations", "priority": -1, "content": "Transformations are algorithms that apply an operation to each element in a set of (zero or more) input ranges and then stores the result in a destination range. One example we have already seen is thrust::fill , which sets all elements of a range to a specified value. Other transformations include thrust::sequence , thrust::replace , and of course thrust::transform . Refer to the documentation for a complete listing. The following source code demonstrates several of the transformation algorithms. Note that thrust::negate and thrust::modulus are known as functors in C++ terminology. Thrust provides these and other common functors like plus and multiplies in the file thrust/functional.h . #include &lt;thrust/device_vector.h&gt; #include &lt;thrust/transform.h&gt; #include &lt;thrust/sequence.h&gt; #include &lt;thrust/copy.h&gt; #include &lt;thrust/fill.h&gt; #include &lt;thrust/replace.h&gt; #include &lt;thrust/functional.h&gt; #include &lt;iostream&gt; int main(void) { // allocate three device_vectors with 10 elements thrust::device_vector&lt;int&gt; X(10); thrust::device_vector&lt;int&gt; Y(10); thrust::device_vector&lt;int&gt; Z(10); // initialize X to 0,1,2,3, .... thrust::sequence(X.begin(), X.end()); // compute Y = -X thrust::transform(X.begin(), X.end(), Y.begin(), thrust::negate&lt;int&gt;()); // fill Z with twos thrust::fill(Z.begin(), Z.end(), 2); // compute Y = X mod 2 thrust::transform(X.begin(), X.end(), Z.begin(), Y.begin(), thrust::modulus&lt;int&gt;()); // replace all the ones in Y with tens thrust::replace(Y.begin(), Y.end(), 1, 10); // print Y thrust::copy(Y.begin(), Y.end(), std::ostream_iterator&lt;int&gt;(std::cout, &quot;\\n&quot;)); return 0; } While the functors in thrust/functional.h cover most of the built-in arithmetic and comparison operations, we often want to do something different. For example, consider the vector operation y &lt;- a * x + y where x and y are vectors and a is a scalar constant. This is the well-known SAXPY operation provided by any BLAS library. If we want to implement SAXPY with Thrust we have a few options. The first is to use two transformations (one addition and one multiplication) and a temporary vector filled with the value a . A better choice is to use a single transformation with a user-defined functor that does exactly what we want. We illustrate both approaches in the source code below. struct saxpy_functor { const float a; saxpy_functor(float _a) : a(_a) {} __host__ __device__ float operator()(const float&amp; x, const float&amp; y) const { return a * x + y; } }; void saxpy_fast(float A, thrust::device_vector&lt;float&gt;&amp; X, thrust::device_vector&lt;float&gt;&amp; Y) { // Y &lt;- A * X + Y thrust::transform(X.begin(), X.end(), Y.begin(), Y.begin(), saxpy_functor(A)); } void saxpy_slow(float A, thrust::device_vector&lt;float&gt;&amp; X, thrust::device_vector&lt;float&gt;&amp; Y) { thrust::device_vector&lt;float&gt; temp(X.size()); // temp &lt;- A thrust::fill(temp.begin(), temp.end(), A); // temp &lt;- A * X thrust::transform(X.begin(), X.end(), temp.begin(), temp.begin(), thrust::multiplies&lt;float&gt;()); // Y &lt;- A * X + Y thrust::transform(temp.begin(), temp.end(), Y.begin(), Y.begin(), thrust::plus&lt;float&gt;()); } Both saxpy_fast and saxpy_slow are valid SAXPY implementations, however saxpy_fast will be significantly faster than saxpy_slow . Ignoring the cost of allocating the temp vector and the arithmetic operations we have the following costs: fast_saxpy : performs 2N reads and N writes slow_saxpy : performs 4N reads and 3N writes Since SAXPY is memory bound (its performance is limited by memory bandwidth, not floating point performance) the larger number of reads and writes makes saxpy_slow much more expensive. In contrast, saxpy_fast will perform about as fast as SAXPY in an optimized BLAS implementation. In memory bound algorithms like SAXPY it is generally worthwhile to apply kernel fusion (combining multiple operations into a single kernel) to minimize the number of memory transactions. thrust::transform only supports transformations with one or two input arguments (e.g. f(x)\\rightarrow y and f(x,x)\\rightarrow y ). When a transformation uses more than two input arguments it is necessary to use a different approach. The arbitrary_transformation example demonstrates a solution that uses thrust::zip_iterator and thrust::for_each . LegacyDitaAnchor reductions", "keywords": []}, {"id": 20, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#vectors", "display_name": "Vectors", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "vectors", "priority": -1, "content": "Thrust provides two vector containers, host_vector and device_vector . As the names suggest, host_vector is stored in host memory while device_vector lives in GPU device memory. Thrust\u2019s vector containers are just like std::vector in the C++ STL . Like std::vector , host_vector and device_vector are generic containers (able to store any data type) that can be resized dynamically. The following source code illustrates the use of Thrust\u2019s vector containers. #include &lt;thrust/host_vector.h&gt; #include &lt;thrust/device_vector.h&gt; #include &lt;iostream&gt; int main(void) { // H has storage for 4 integers thrust::host_vector&lt;int&gt; H(4); // initialize individual elements H[0] = 14; H[1] = 20; H[2] = 38; H[3] = 46; // H.size() returns the size of vector H std::cout &lt;&lt; &quot;H has size &quot; &lt;&lt; H.size() &lt;&lt; std::endl; // print contents of H for(int i = 0; i &lt; H.size(); i++) std::cout &lt;&lt; &quot;H[&quot; &lt;&lt; i &lt;&lt; &quot;] = &quot; &lt;&lt; H[i] &lt;&lt; std::endl; // resize H H.resize(2); std::cout &lt;&lt; &quot;H now has size &quot; &lt;&lt; H.size() &lt;&lt; std::endl; // Copy host_vector H to device_vector D thrust::device_vector&lt;int&gt; D = H; // elements of D can be modified D[0] = 99; D[1] = 88; // print contents of D for(int i = 0; i &lt; D.size(); i++) std::cout &lt;&lt; &quot;D[&quot; &lt;&lt; i &lt;&lt; &quot;] = &quot; &lt;&lt; D[i] &lt;&lt; std::endl; // H and D are automatically deleted when the function returns return 0; } As this example shows, the = operator can be used to copy a host_vector to a device_vector (or vice-versa). The = operator can also be used to copy host_vector to host_vector or device_vector to device_vector . Also note that individual elements of a device_vector can be accessed using the standard bracket notation. However, because each of these accesses requires a call to cudaMemcpy , they should be used sparingly. We\u2019ll look at some more efficient techniques later. It\u2019s often useful to initialize all the elements of a vector to a specific value, or to copy only a certain set of values from one vector to another. Thrust provides a few ways to do these kinds of operations. #include &lt;thrust/host_vector.h&gt; #include &lt;thrust/device_vector.h&gt; #include &lt;thrust/copy.h&gt; #include &lt;thrust/fill.h&gt; #include &lt;thrust/sequence.h&gt; #include &lt;iostream&gt; int main(void) { // initialize all ten integers of a device_vector to 1 thrust::device_vector&lt;int&gt; D(10, 1); // set the first seven elements of a vector to 9 thrust::fill(D.begin(), D.begin() + 7, 9); // initialize a host_vector with the first five elements of D thrust::host_vector&lt;int&gt; H(D.begin(), D.begin() + 5); // set the elements of H to 0, 1, 2, 3, ... thrust::sequence(H.begin(), H.end()); // copy all of H back to the beginning of D thrust::copy(H.begin(), H.end(), D.begin()); // print D for(int i = 0; i &lt; D.size(); i++) std::cout &lt;&lt; &quot;D[&quot; &lt;&lt; i &lt;&lt; &quot;] = &quot; &lt;&lt; D[i] &lt;&lt; std::endl; return 0; } Here we\u2019ve illustrated use of the fill , copy , and sequence functions. The copy function can be used to copy a range of host or device elements to another host or device vector. Like the corresponding STL function, thrust::fill simply sets a range of elements to a specific value. Thrust\u2019s sequence function can be used to create a sequence of equally spaced values. LegacyDitaAnchor thrust-namespace", "keywords": []}, {"id": 21, "doc_id": 22, "filename": "index.html", "domain_name": "page", "name": "index#zip-iterator", "display_name": "zip_iterator", "type": "section", "display_type": "Page section", "docname": "index", "anchor": "zip-iterator", "priority": -1, "content": "Keep reading, we\u2019ve saved the best iterator for last! The zip_iterator is an extremely useful gadget: it takes multiple input sequences and yields a sequence of tuples. In this example we \u201czip\u201d together a sequence of int and a sequence of char into a sequence of tuple&lt;int,char&gt; and compute the tuple with the maximum value. #include &lt;thrust/iterator/zip_iterator.h&gt; ... // initialize vectors thrust::device_vector&lt;int&gt; A(3); thrust::device_vector&lt;char&gt; B(3); A[0] = 10; A[1] = 20; A[2] = 30; B[0] = &#x27;x&#x27;; B[1] = &#x27;y&#x27;; B[2] = &#x27;z&#x27;; // create iterator (type omitted) first = thrust::make_zip_iterator(thrust::make_tuple(A.begin(), B.begin())); last = thrust::make_zip_iterator(thrust::make_tuple(A.end(), B.end())); first[0] // returns tuple(10, &#x27;x&#x27;) first[1] // returns tuple(20, &#x27;y&#x27;) first[2] // returns tuple(30, &#x27;z&#x27;) // maximum of [first, last) thrust::maximum&lt; tuple&lt;int,char&gt; &gt; binary_op; thrust::tuple&lt;int,char&gt; init = first[0]; thrust::reduce(first, last, init, binary_op); // returns tuple(30, &#x27;z&#x27;) What makes zip_iterator so useful is that most algorithms accept either one, or occasionally two, input sequences. The zip_iterator allows us to combine many independent sequences into a single sequence of tuples, which can be processed by a broad set of algorithms. Refer to the arbitrary_transformation example to see how to implement a ternary transformation with zip_iterator and for_each . A simple extension of this example would allow you to compute transformations with multiple output sequences as well. In addition to convenience, zip_iterator allows us to implement programs more efficiently. For example, storing 3d points as an array of float3 in CUDA is generally a bad idea, since array accesses are not properly coalesced. With zip_iterator we can store the three coordinates in three separate arrays, which does permit coalesced memory access. In this case, we use zip_iterator to create a virtual array of 3d vectors which we can feed in to Thrust algorithms. Refer to the dot_products_with_zip example for additional details. LegacyDitaAnchor additional-resources", "keywords": []}, {"id": 22, "doc_id": 22, "filename": "index.html", "domain_name": "std", "name": "index", "display_name": "Introduction", "type": "doc", "display_type": "Page", "docname": "index", "anchor": "", "priority": -1, "content": "Thrust The API reference guide for Thrust, the CUDA C++ template library. LegacyDitaAnchor introduction Thrust is a C++ template library for CUDA based on the Standard Template Library (STL). Thrust allows you to implement high performance parallel applications with minimal programming effort through a high-level interface that is fully interoperable with CUDA C. Thrust provides a rich collection of data parallel primitives such as scan, sort, and reduce, which can be composed together to implement complex algorithms with concise, readable source code. By describing your computation in terms of these high-level abstractions you provide Thrust with the freedom to select the most efficient implementation automatically. As a result, Thrust can be utilized in rapid prototyping of CUDA applications, where programmer productivity matters most, as well as in production, where robustness and absolute performance are crucial. This document describes how to develop CUDA applications with Thrust. The tutorial is intended to be accessible, even if you have limited C++ or CUDA experience. LegacyDitaAnchor installation-and-versioning", "keywords": []}]};